/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*-----------------------------------------------------------------------------------------------*
 * Description: ANSI C library porting layer
 *              Note, this file CANNOT be use with libc/stdio.h
 *-----------------------------------------------------------------------------------------------*/

#ifndef __ANSI_LIB_H
#define __ANSI_LIB_H

/* ARM tool chain header */
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>

#ifdef LINUX_TURNKEY_SOLUTION
    #include <stdio.h>
    #include <ctype.h>
    #include <time.h>
    #include <errno.h>
    #include <sys/time.h>
#else /* this is for Nucleus */

#ifdef POSIX_SUPPORT
    #include <stdio.h>
#endif

/* original in sys/type.h but we do not have */
#ifndef __SSIZE_T_
#define __SSIZE_T_
    typedef int ssize_t;
#endif

/* original in stdio.h */
#ifndef POSIX_SUPPORT
    #define BUFSIZ  1024
#endif

/* middleware header file */
#include "x_os.h"
#include "x_dbg.h"

#include "x_net_rand.h"

/**
 * struct timeval, the arm tool chain do not have this 
 * we define it in libc/u_time.h  
 */
#ifdef MT8520
#include "u_time.h"
#else
#ifndef LINUX_TURNKEY_SOLUTION
#include "time.h"
#else
#include <time.h>
#endif
#endif

/* in ctype.h the follow is lowcase */
#define ISXDIGIT        isxdigit
#define ISDIGIT         isdigit
//#define ISSPACE         isspace
#define ISSPACE(c)      ((c) == ' ' || ((c) >= 0x09 && (c) <= 0x0D))//haifei change it for vincent.
#define ISALPHA         isalpha
#define ISALNUM         isalnum

/* system call isdigit now has bug. so use this macro to replace.  */
#ifdef isdigit
#undef isdigit
#endif
#define isdigit(c)    ((c) >='0' && (c) <= '9'? 1:0)


/* RVCT 2.2/2.1 does not include these APIs, use middleware API instead of it */
//#ifndef POSIX_SUPPORT
    //#define strcasecmp      x_strcasecmp
    //#define strncasecmp     x_strncasecmp
//#endif

/* network module provide rand like API */
#define srand           x_net_rand_set_seed
#define rand            x_net_rand_get



/* some of stdio.h api declare... tool chain's stdio.h conflict with libc/stdio.h */
#ifndef POSIX_SUPPORT
	#define SEEK_SET       ((UINT8) 1) /*add by mtk40135*/
	#define SEEK_CUR       ((UINT8) 2) /*add by mtk40135*/
	#define SEEK_END       ((UINT8) 3) /*add by mtk40135*/
    #define FILE            void
    #define stdin           ((void*)0)
    #define stdout          ((void*)1)
    #define stderr          ((void*)1)

    extern int              sscanf(const char *s, const char * format, ... );
    extern int              snprintf(char *s, size_t n, const char * format, ...);
    extern int              sprintf(char * s, const char * format, ...);
    extern int              vprintf(const char * format, va_list ap);
    extern int              vsnprintf(char * s, size_t n, const char * format, va_list ap);
    extern int              vsprintf(char * s, const char * format, va_list ap);

    /* replace stdio api to our glue layer */
    extern void *           _glue_fopen(const char *filename, const char *mode);
    extern int              _glue_feof(void *stream);
    extern int              _glue_fseek(void *stream, long offset, int whence);
    extern size_t           _glue_fread(void *ptr, size_t size, size_t nitems, void *stream);
    extern long             _glue_ftell(void *stream);
    extern size_t           _glue_fwrite(const void *buffer, size_t size, size_t count, void *stream);
    extern int              _glue_fputc(int c, void *stream);
    extern int              _glue_fputs(const char* ps_str, void *stream);
    extern int              _glue_fgetc(void *stream);
    extern int              _glue_fgets(char* ps_str, int n, void *stream);
    extern int              _glue_fprintf(void *stream, const CHAR *ps_format, ...);
    extern int              _glue_fclose(void *stream);
    extern int              _glue_fflush(void *stream);
    extern int              _glue_file_size(const char *file_name);
    extern int              _glue_ferror(void *stream);
    extern int              _glue_getuid(void);
    extern int              _glue_vfprintf(void *stream, const char *format, va_list ap);
    extern int              _glue_stat(const char *file_name, void *buf);

    /* glue stdio.h to our glue api */
    #define fopen           _glue_fopen
    #define feof            _glue_feof
    #define fseek           _glue_fseek
    #define fread           _glue_fread 
    #define ftell           _glue_ftell 
    #define fwrite          _glue_fwrite
    #define fputc           _glue_fputc
    #define fputs           _glue_fputs
    #define fgetc           _glue_fgetc
    #define fgets           _glue_fgets
    #define fprintf         _glue_fprintf
    #define fclose          _glue_fclose
    #define ferror          _glue_ferror
    #define fflush          _glue_fflush
    #define vfprintf        _glue_vfprintf
    #define stat            _glue_stat
#endif

/* 
 * TV use RVCT 2.1 which sscanf cause system crash 
 * in case BD use more newer RVCT and it's work fine.
 * So, when TV use, use x_sscanf.
*/
#ifndef MT8520
    #define sscanf      x_sscanf
#endif
    
/* something wrong on RVCT 2.1's atoi. */
#ifndef MT8520
    #define atoi(str)   ((int)x_strtoll(str, NULL, 10))
    #define strtoll(pc_beg_ptr, ppc_end_ptr, ui1_base)   ((int)x_strtoll(pc_beg_ptr, ppc_end_ptr, ui1_base))
    #define strtoul(pc_beg_ptr, ppc_end_ptr, ui1_base)   ((int)x_strtoll(pc_beg_ptr, ppc_end_ptr, ui1_base))
#endif

/* for stat use. */
#ifndef POSIX_SUPPORT
    struct _glue_stat
    {
        CHAR value[100];
    };
#endif

/* unix relate */
#define uid_t           int
#define getuid          _glue_getuid

#endif /*__ANSI_LIB_H*/
#endif /* Nucleus c tool chain porting layer */
