/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*----------------------------------------------------------------------------*/
/*! @file x_sm_cc_isdb_hdlr.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/11/26 $
 *  $Author: dtvbm11 $
 *  
 *  @par Description:
 *         This header file contains ISDB closed caption Stream handler specific 
 *         definitions, which are exported to applications.
 */
/*----------------------------------------------------------------------------*/

#ifndef _X_SM_CC_ISDB_HDLR_H_
#define _X_SM_CC_ISDB_HDLR_H_

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_STRM_MNGR_CCI_HDLR
*  @{  
*/
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
#include "u_common.h"
#include "u_sm_cc_isdb_hdlr.h"


#define CC_ISDB_INIT_OK                 ((INT32) 0)    /**< The routine was successful */
#define CC_ISDB_INIT_FAIL               ((INT32) -1)    /**< The routine was fail */
extern BOOL  fg_g_cc_isdb_raw_data_is_utf8;


/*------------------------------------------------------------------*/
/*! @struct SM_CCI_CONFIG_T
 *  @brief  This structure contains the configuration data for ISDB closed caption stream handler
 *  @code
 *  typedef struct
 *  {
 *      UINT8                   ui1_wgl_order;
 *  }   SM_CCI_CONFIG_T;
 *  @endcode
 *  @li@c  ui1_wgl_order                                     - ISDB closed caption widget app order
 */
/*------------------------------------------------------------------*/
typedef struct
{
    UINT8                   ui1_wgl_order;
}   SM_CCI_CONFIG_T;

#if 0
typedef struct SM_CCI_HDLR_T
{
    
    HANDLE_T                h_hdlr_handle; 
    SM_CCI_SYS_SERVICE      t_sys_svc;  /* system serivce */
    HANDLE_T                h_wgl_app;
    UINT8                   ui1_wgl_order;
    
    SM_CCI_INSTANCE_T       at_cci_inst[SM_CCI_INST_TYPE_MODULE_NUM];
    //UINT8*                  pui1_demux_buf;/* demux buffer, both superimpose and caption will use the buffer */
}   SM_CCI_HDLR_T;
#endif

//SM_CCI_HDLR_T    t_g_cci_hdlr;

/*-----------------------------------------------------------------------------
                    functions declarations
-----------------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
/*! @brief Initialize the ISDB closed caption configuration
 *  @param [in]     pt_config                   - Configuration context
 *  @param [out]
 *  @note
 *  @see
 *  @return     Return the error code.
 *  @retval      CC_ISDB_INIT_OK        - The routine was successful.
 *  @retval      CC_ISDB_INIT_FAIL      - The routine was fail.
 */
/*------------------------------------------------------------------*/
extern INT32 x_sm_cc_isdb_hdlr_init( SM_CCI_CONFIG_T*   pt_config );  


/*------------------------------------------------------------------*/
/*!@brief  This function is used to set the ISDB closed caption attributes.
 *  @param [in]    t_cci_att                    - ISDB CC attributes
 */
/*------------------------------------------------------------------*/
extern VOID x_sm_cc_isdb_hdlr_set_att( SM_CCI_ATTRIBS_T  t_cci_att );  

/*------------------------------------------------------------------*/
/*!@brief  This function is used to get the ISDB closed caption raw data is utf8?
 *  @param [in]    fg_is_utf8_data      - The utf8 flag
 */
/*------------------------------------------------------------------*/
extern VOID x_sm_cc_isdb_raw_data_is_utf8(BOOL  fg_is_utf8_data );

#endif /* _X_SM_CC_ISDB_HDLR_H_ */

/*----------------------------------------------------------------------------*/
/*! @} */                                            /* end of groupMW_STRM_MNGR_CCI_HDLR */
/*----------------------------------------------------------------------------*/



